<?php
/**
 * @Author: Sebastian Castro
 * @Date:   2017-03-28 15:29:03
 * @Last Modified by:   Sebastian Castro
 * @Last Modified time: 2017-09-22 10:31:02
 */

namespace App\Admin;

use Sonata\AdminBundle\Admin\AbstractAdmin;
use Sonata\AdminBundle\Datagrid\DatagridMapper;
use Sonata\AdminBundle\Datagrid\ListMapper;
use Sonata\AdminBundle\Form\FormMapper;
use Sonata\AdminBundle\Route\RouteCollection;
use Sonata\FormatterBundle\Form\Type\SimpleFormatterType;

class WrapperAdmin extends AbstractAdmin
{
    protected $datagridValues = [
        '_page' => 1,
        '_sort_order' => 'ASC',
        '_sort_by' => 'position',
    ];

    protected function configureFormFields(FormMapper $formMapper)
    {
        $formMapper->add('title', null);
        $formMapper->add('content', SimpleFormatterType::class, [
                'format' => 'richhtml',
                'label' => 'Contenu du bandeau',
                'required' => false,
                'ckeditor_context' => 'full',
            ]);
        $formMapper->add('rawContent', null, [
                'label' => 'Contenu en raw html (optionel)',
                'required' => false,
                'attr' => ['class' => 'gogo-code-editor', 'format' => 'html', 'height' => '150'],
            ]);
        $formMapper->add('textColor', null, ['required' => false, 'attr' => ['class' => 'gogo-color-picker']]);
        $formMapper->add('backgroundColor', null, ['required' => false, 'attr' => ['class' => 'gogo-color-picker']]);
    }

    protected function configureDatagridFilters(DatagridMapper $datagridMapper)
    {
        $datagridMapper->add('title');
    }

    protected function configureListFields(ListMapper $listMapper)
    {
        $listMapper
            ->addIdentifier('title')
            ->add('_action', 'actions', [
                'actions' => [
                    'show' => [],
                    'edit' => [],
                    'delete' => [],
                    'move' => [
                        'template' => '@PixSortableBehavior/Default/_sort.html.twig',
                    ],
                ],
            ]);
    }

    protected function configureRoutes(RouteCollection $collection)
    {
        $collection->add('move', $this->getRouterIdParameter().'/move/{position}');
    }
}
